#include "dsp/benchmark/bench.h"
#include "image/image.h"

namespace satdump
{
    namespace ndsp
    {
        image::Image renderResults(std::vector<DSPBenchResult> results);
    }
} // namespace satdump